"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _spResource = require("sp-resource");

var _spResource2 = _interopRequireDefault(_spResource);

var _winston = require("winston");

var _winston2 = _interopRequireDefault(_winston);

var _nodeUuid = require("node-uuid");

var _uuidValidate = require("uuid-validate");

var _uuidValidate2 = _interopRequireDefault(_uuidValidate);

var _spClient = require("sp-client");

var _url = require("url");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var User = function (_Resource) {
  _inherits(User, _Resource);

  function User() {
    _classCallCheck(this, User);

    return _possibleConstructorReturn(this, (User.__proto__ || Object.getPrototypeOf(User)).apply(this, arguments));
  }

  _createClass(User, [{
    key: "auth",
    value: function auth(ctx, doc) {
      return Promise.resolve();
    }
  }, {
    key: "authUpdate",
    value: function authUpdate(ctx, doc, newDoc) {
      if (newDoc.handle.indexOf("new-user") !== -1) {
        throw new _spResource2.default.APIError(422, "Your handle may not contain 'new-user'.");
      }
    }

    // Nope. Happens through the context method further down.

  }, {
    key: "authCreate",
    value: function authCreate(ctx, doc) {
      throw new _spResource2.default.APIError(403, "You may not create users");
    }

    // Also nope.

  }, {
    key: "authDelete",
    value: function authDelete(ctx, doc) {
      throw new _spResource2.default.APIError(403, "You may not delete users");
    }
  }, {
    key: "authQuery",
    value: function authQuery(ctx, query) {
      return Promise.resolve({});
    }
  }, {
    key: "default",
    value: function _default() {
      return _get(User.prototype.__proto__ || Object.getPrototypeOf(User.prototype), "default", this).call(this).then(function (doc) {
        return _extends({}, doc, {
          id: (0, _nodeUuid.v4)(), // Necessary because authToken is our primary key
          handle: "new-user-" + Math.round(Math.random() * 1000000)
        });
      });
    }

    /**
     * Used upon new incoming traffic to create new users for folks as needed.
     */

  }, {
    key: "findOrCreateFromContext",
    value: function findOrCreateFromContext(ctx) {
      var _this2 = this;

      if (!ctx.jwt || typeof ctx.jwt.sub !== "string") {
        throw new Error("don't have ctx.jwt.sub");
      }
      // TODO this should really be thought through more... for now this handles the normalization
      // of auth0 tokens to streamplace tokens.
      var identity = ctx.jwt.sub;

      return this.db.find(ctx, { identity: identity }).then(function (users) {
        if (users.length === 1) {
          return Promise.resolve(users[0]);
        }
        if (users.length > 1) {
          throw new Error("Data integrity violation: multiple users with identity " + identity);
        }

        // Okay, zero matches, need to create a user. By this point, the API server has made sure
        // that we have something of the form https://stream.place/users/uuid-goes-here, so let's
        // sanity check that and proceed.
        var identity = ctx.jwt.sub;

        var _parseUrl = (0, _url.parse)(identity),
            path = _parseUrl.path;

        var _path$split$filter = path.split("/").filter(function (s) {
          return s !== "";
        }),
            _path$split$filter2 = _toArray(_path$split$filter),
            apiPath = _path$split$filter2[0],
            usersPath = _path$split$filter2[1],
            newId = _path$split$filter2[2],
            rest = _path$split$filter2.slice(3);

        if (apiPath !== "api" || usersPath !== "users" || !newId || rest.length > 0) {
          throw new Error("Unknown form for user identity: " + ctx.jwt.sub);
        }
        var newUser = void 0;
        return _this2.default().then(function (def) {
          newUser = _extends({}, def, {
            identity: identity,
            roles: []
          });
          if ((0, _uuidValidate2.default)(newId)) {
            newUser.id = newId; // Regular users already have one, services get a random one
          } else if (ctx.jwt.roles && ctx.jwt.roles.includes("SERVICE")) {
            newUser.roles = ctx.jwt.roles;
            newUser.handle = newId;
          } else {
            _winston2.default.error("MAYDAY we're supposed to create a user but it's not a service and doesn't have a UUID?");
            throw new Error("User creation failed");
          }
          return _this2.validate(ctx, newUser);
        }).then(function () {
          return new Promise(function (resolve, reject) {
            // Avoid a concurrency problem -- if multiple attempts happen, only let one succeed.
            _this2.db.insert(ctx, newUser).then(function (user) {
              _winston2.default.info("Created user " + user.handle + " (" + user.id + ") for identity " + identity);
              resolve(user);
            }).catch(function (err) {
              if (err.message.indexOf("Duplicate primary key") === -1) {
                return reject(err);
              }
              // Okay, someone else created it, let's look at theirs...
              _this2.db.find(ctx, { identity: identity }).catch(reject).then(function (docs) {
                if (docs.length === 1) {
                  return resolve(docs[0]);
                }
                throw new Error("MAYDAY: can't create a user, can't find a user?!?");
              });
            });
          });
        }).then(function (user) {
          return user;
        });
      });
    }
  }]);

  return User;
}(_spResource2.default);

exports.default = User;


User.primaryKey = "identity";
User.indices = ["id"];